/*
   FNT - nodelist parser
   Function for manipulating with file

   Copyright (C) 2009 Free Software Foundation, Inc.

   Written by:
   Slava Zanko <slavazanko@gmail.com>


   This file is part of the FIDO New Technology.

   FIDO New Technology is free software: you can redistribute it and/or 
   modify it under the terms of the GNU General Public License as published 
   by the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifdef HAVE_CONFIG_H
#	include <config.h>
#endif
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>

#include "../include/global.h"
#include "../include/types.h"
#include "../include/error.h"

#include "file_operations.h"

/*** global variables **************************************************/

/*** file scope macro definitions **************************************/

/*** file scope type declarations **************************************/

/*** file scope variables **********************************************/

static char *tmp_work_dir = NULL;
static char *tmp_file = NULL;

/*** file scope functions **********************************************/

static int
ndl_fileop_opnfl (char *flname)
{
    int file;

    if (flname == NULL)
    {
	fnt_error_push (0, 0, _("ERROR: No any files specified\n"));
	return -1;
    }
    if (!g_file_test (flname, G_FILE_TEST_EXISTS))
    {
	fnt_error_push (0, 0, _("ERROR: File not exists: %s\n"), flname);
	return -1;
    }
    file = open (flname, O_RDONLY);
    if (file == -1)
    {
	fnt_error_push (0, 0,
			_("ERROR: Unable to open file %s for reading\n"),
			flname);
	return -1;
    }
    return file;
}

static char *
ndl_fileop_make_fullPath (const char *flname)
{
    char *curr_dir;
    char *full_path;

    if (g_path_is_absolute (flname))
	return g_strdup (flname);

    curr_dir = g_get_current_dir ();
    full_path = g_build_filename (curr_dir, flname, NULL);
    g_free (curr_dir);
    return full_path;
}

static char *
ndl_fileop_getTempDirname (const char *some_data)
{
    char *ret_str, *checksumm;

    checksumm =
	g_compute_checksum_for_string (G_CHECKSUM_SHA256, some_data, -1);

    ret_str = g_build_filename (g_get_tmp_dir (), "fnt-ndl", checksumm, NULL);

    g_free (checksumm);

    return ret_str;
}

static void
ndl_fileop_delete_dir (const char *dir)
{
    GDir *dh;
    const gchar *file;
    gchar *full_path;
    dh = g_dir_open (dir, 0, NULL);
    if (dh == NULL)
	return;
    while ((file = g_dir_read_name (dh)) != NULL)
    {
	full_path = g_build_filename (dir, file, NULL);

	if (g_file_test (full_path, G_FILE_TEST_IS_DIR))
	{
	    ndl_fileop_delete_dir (full_path);
	}
	else
	    g_unlink (full_path);

	g_free (full_path);
    }
    g_dir_close (dh);
    g_rmdir (dir);
}

static void
ndl_fileop_unpack_file (const char *command)
{
    GDir *dh;
    const gchar *file;
    gchar *full_path;

    char *curr_dir = g_get_current_dir ();

    tmp_work_dir = ndl_fileop_getTempDirname (command);

    if (g_file_test (tmp_work_dir, G_FILE_TEST_IS_DIR))
	ndl_fileop_delete_dir (tmp_work_dir);

    g_mkdir_with_parents (tmp_work_dir, 0700);

    chdir (tmp_work_dir);
    system (command);
    chdir (curr_dir);
    g_free (curr_dir);

    dh = g_dir_open (tmp_work_dir, 0, NULL);

    if (dh == NULL)
	return;

    while ((file = g_dir_read_name (dh)) != NULL)
    {
	full_path = g_build_filename (tmp_work_dir, file, NULL);
	if (g_file_test (full_path, G_FILE_TEST_IS_DIR))
	{
	    g_free (full_path);
	    continue;
	}
	tmp_file = full_path;
	break;
    }
    g_dir_close (dh);
}


static void
ndl_fileop_detect (int *file, const char *flname)
{
    char *cmd = NULL;
    char *full_flname;
    char read_buff[8];
    char sign_zip[] = {
	0x50, 0x4B, 0x03, 0x04, 0x14, 0x00
    };
    char sign_7z[] = {
	0x37, 0x7A, 0xBC, 0xAF, 0x27, 0x1C, 0x00
    };
    char sign_gz[] = {
	0x1F, 0x8B, 0x08
    };

    char sign_bzip2[] = {
	0x42, 0x5A, 0x68, 0x39, 0x31, 0x41, 0x59, 0x26
    };

    if (read (*file, read_buff, 8) == -1)
    {
	close (*file);
	*file = -1;
	fnt_error_push (0, 0, _("ERROR: Unable to read from file %s\n"),
			flname);
	return;
    }
    lseek (*file, 0, SEEK_SET);
    /* check if file plain text */
    if (read_buff[0] == ';')
	return;

    full_flname = ndl_fileop_make_fullPath (flname);

    /* FIXME! Have *NIX-specific commands! */

    /* check if file is a zip-arhive */
    if (!memcmp (read_buff, sign_zip, sizeof (sign_zip)))
    {
	cmd = g_strdup_printf ("unzip %s", full_flname);
    }
    /* check if file is a 7z-arhive */
    else if (!memcmp (read_buff, sign_7z, sizeof (sign_7z)))
    {
	cmd = g_strdup_printf ("7z x %s", full_flname);
    }
    /* check if file is a gz-arhive */
    else if (!memcmp (read_buff, sign_gz, sizeof (sign_gz)))
    {
	cmd = g_strdup_printf ("cat %s | gzip -cd > file.txt", full_flname);
    }
    /* check if file is a bzip2-arhive */
    else if (!memcmp (read_buff, sign_bzip2, sizeof (sign_bzip2)))
    {
	cmd = g_strdup_printf ("cat %s | bzip2 -cd > file.txt", full_flname);
    }
    /* Unknown format */
    else
    {
	close (*file);
	*file = -1;
	fnt_error_push (0, 0, _("ERROR: Unknown file format: %s\n"), flname);
	return;
    }
    g_free (full_flname);

    if (cmd == NULL)
	return;

    ndl_fileop_unpack_file (cmd);
    g_free (cmd);

    if (tmp_file == NULL)
	return;

    close (*file);
    *file = ndl_fileop_opnfl (tmp_file);

    if (read (*file, read_buff, 8) == -1)
    {
	close (*file);
	*file = -1;
	fnt_error_push (0, 0, _("ERROR: Unable to read from file %s\n"),
			flname);
	return;
    }
    lseek (*file, 0, SEEK_SET);
    /* check if file plain text */
    if (read_buff[0] != ';')
    {
	fnt_error_push (0, 0, _("ERROR: Unknown file format: %s\n"), flname);
	close (*file);
	*file = -1;
    }

}

/*** public functions **************************************************/

int
ndl_fileop_openfile (char *flname)
{
    int file;
    file = ndl_fileop_opnfl (flname);
    if (file == -1)
	return -1;

    ndl_fileop_detect (&file, flname);
    if (file == -1)
	return -1;

    return file;
}

void
ndl_fileop_closefile (int file)
{
    close (file);

    if (tmp_file != NULL)
	g_free (tmp_file);

    if (tmp_work_dir != NULL)
    {
	ndl_fileop_delete_dir (tmp_work_dir);
	g_free (tmp_work_dir);
    }

}
